﻿using System;
using EnvelopeCreatorProject.Interfaces;
using log4net;
using EnvelopeCreatorProject.Models;
using EnvelopeCreatorProject.Models.Signers;
using System.Linq;
using EnvelopeCreatorProject.Exceptions;

namespace EnvelopeCreatorProject
{
    public class EnvelopeTransporterCreator : IEnvelopeCreator
    {
        private readonly ILog logger = LogManager.GetLogger("EnvelopeCreator");
        private readonly string _integratorKey;
        private readonly string _APIVersion;

        public EnvelopeTransporterCreator(string integratorKey, string APIVersion)
        {
            _integratorKey = integratorKey;
            _APIVersion = APIVersion;
        }


        public EnvelopeResult CreateEnvelope(AccountAuthentication authentication, EnvelopeData EData)
        {
            try
            {
                ISigner signer = GetSigner(authentication);
                return signer.GenerateEnvelope(EData);
            }
            catch (OAuthTokenAuthorizationException ote)
            {
                return new EnvelopeResult()
                {
                    ErrorCode = ote.ErrorCode,
                    Message = ote.Message
                };
            }
            catch (Exception ex)
            {
                logger.Error(" -- CreateEnvelope error -- MESSAGE: " + ex.Message + ". STACK TRACE: " +
                             ex.StackTrace.ToString());
                return new EnvelopeResult()
                {
                    ErrorCode = 999,
                    Message = "Unexpected error occurred. Message: " + ex.Message
                };
            }
        }

        private ISigner GetSigner(AccountAuthentication authentication)
        {
            var tokenHandler = new DocusignTokenHandler(_integratorKey, _APIVersion);
            var senderAccountInfo = tokenHandler.getAccountInformation(authentication.SenderToken, authentication.EnvironmentURL);
            var authAccountInfo = tokenHandler.getAccountInformation(authentication.AuthToken, authentication.EnvironmentURL);
            var authBaseURl = authAccountInfo == null ? "" : authAccountInfo.loginAccounts.First().baseUrl;
            var senderEmail = senderAccountInfo == null ? "" : senderAccountInfo.loginAccounts.First().email;

            return authentication.SOBOFlag ?
                (OAuthSigner)new SOBOSigner() { AuthToken = authentication.AuthToken, BaseURL = authBaseURl, SenderMail = senderEmail, SenderToken = authentication.SenderToken }
                : (OAuthSigner)new StandardSigner() { AuthToken = authentication.AuthToken, BaseURL = authBaseURl };
        }
    }
}